#ifndef SR_API_ERRORCODES_H
#define SR_API_ERRORCODES_H


/**
 *	General defines for Errors
 */
#define SUCCESS								0			/*!< function was successful */
#define	FOUND_STRING						1			/*!< string was found */
#define	FOUND_INTEGER						2			/*!< integer was found */
#define	Z_OVERFLOW							3			/*!< Z has an overflow */
#define	FOUND_FLOAT							4			/*!< float was found */
#define	DIFFERENT							5			/*!< values are different */
#define EQUAL								6			/*!< values are equal */
#define	CONNECTION_ESTABLISHED				7			/*!< connection to the camera established */


#define ERR_CD_NULLPOINTER					-1			/*!< camera descriptor is a nullpointer */
#define ERR_NULLPOINTER						-2			/*!< nullpointer exception */
#define	ERR_ALLOCATION_FAILED				-3			/*!< allocate memory failed */
#define	ERR_READ_FILE_FAILED				-4			/*!< file couldn't be read */
#define	ERR_FUNCTION_NOT_SUCCESSFUL			-5			/*!< function wasn't successful */
#define	ERR_WRITE_FILE_FAILED				-6			/*!< write data to file failed */
#define	ERR_FILE_NEVER_FOUND				-7			/*!< no file found */
#define	ERR_INVALID_FILENAME				-8			/*!< invalid filename */
#define	ERR_INVALID_FILETYPE				-9			/*!< invalid filetype */
#define	ERR_INVALID_UPDATEFILE				-10			/*!< file is not a smartray update file */

/**
 *	Special defines for Errors
 */
#define ERR_API_IS_INITIALIZED				-1000		/*!< API is still initialized*/
#define ERR_LESS_NR_OF_CAM_INSTANCES		-1001		/*!< number of instances is to less */
#define ERR_TOO_MANY_NR_OF_CAM_INSTANCES	-1002		/*!< number of instances is to much */
#define ERR_CAM_STILL_ACTIVE				-1003		/*!< camera is still active */
#define	ERR_CAM_NOT_ACTIVE					-1004		/*!< camera is not active */
#define	ERR_CAM_CONN_STILL_OPEN				-1005		/*!< connection to camera can't be closed */
#define	ERR_CAM_NOT_CONNECTED				-1006		/*!< camera is not connected */
#define	ERR_SIZE_MISSMATCH					-1007		/*!< size of the read data doesn't match with the expected size */
#define	ERR_NR_USERCB_RAWDATA_MAX			-1008		/*!< numUserCB_RawData is equal to MAXNUM_SRCB_RAWDATA */
#define	ERR_NR_USERCB_CAMDATA_MAX			-1009		/*!< numUserCB_CamData is equal to MAXNUM_SRCB_CAMDATA */
#define	ERR_NR_USERCB_CAMDATAEXT_MAX		-1010		/*!< numUserCB_DataExt is equal to MAXNUM_SRCB_CAMDATAEXT */
#define	ERR_NR_USERCB_PIXELMODE_MAX			-1011		/*!< numUserCB_PixelMode is equal to MAXNUM_SRCB_PIXELMODE*/
#define	ERR_NO_VALID_CB_TYPE				-1012		/*!< no valid CBtype */
#define	ERR_NO_MPAR_AVAILABLE				-1013		/*!< no MPAR for the selected camera available */
#define	ERR_NO_SYSPAR_AVAILABLE				-1014		/*!< no SYSPAR for the selected camera available */
#define	ERR_IP_OR_PORT_IS_ZERO				-1015		/*!< newport or new IP are zero */
#define ERR_IP_IS_BROADCAST					-1016		/*!< newip contain 4 times the value 255 */
#define	ERR_WIDTH_IS_ZERO					-1017		/*!< the width is zero */
#define	ERR_HEIGHT_IS_ZERO					-1018		/*!< the height is zero */
#define	ERR_CENTERX_RANGE_ERROR				-1019		/*!< centerX is not between 0 and 1280 */
#define	ERR_CENTERY_RANGE_ERROR				-1020		/*!< centerY is not between 0 and 1280 */
#define	ERR_INDEX_LESS_THAN_ZERO			-1021		/*!< index is lesser then zero */
#define	ERR_NO_TRACE_FOUND					-1022		/*!< no trace was found */
#define	ERR_INDEX_GREATER_THAN_MAX			-1023		/*!< index is greater than the maximum */
#define	ERR_CAM_NOT_RUNNING					-1024		/*!< camera is not running */
#define	ERR_WRONG_DATTYPE					-1025		/*!< another dattyp is expected */
#define	ERR_OUT_OF_RANGE					-1026		/*!< values are not in range */
#define	ERR_COORDX_IS_ZERO					-1027		/*!< x coordinate of the calculated scale vektor is zero */
#define	ERR_ANGLE_OUT_OF_RANGE				-1028		/*!< angle is greater than 15 degree, this is to much */
#define	ERR_NO_COORD_GREAT_ENOUGH			-1029		/*!< no y coordinate is greater than NONWORLDMARK */
#define	ERR_LUTWIDTH_RANGE_ERROR			-1030		/*!< lutWidth less than zero or greater than the MAXIMAGERSIZE */
#define	ERR_LUTHEIGHT_RANGE_ERROR			-1031		/*!< lutHeight less than zero or greater than the MAXIMAGERSIZE */
#define	ERR_SEARCH_ITEM_NOT_FOUND			-1032		/*!< item in the LUTinfo not found */
#define	ERR_LUTWIDTH_IS_ZERO				-1033		/*!< the width of the lut is zero */
#define	ERR_LUTHEIGHT_IS_ZERO				-1034		/*!< the height of the lut is zero */
#define	ERR_NO_IMAGE_POINT_FOUND			-1035		/*!< no image point was found */
#define	ERR_RANGE_CHECK						-1036		/*!< range of startx and width is greatern than imager size */
#define	ERR_CR_LUT_NOT_VALID				-1037		/*!< correction lut is not valid */
#define	ERR_CR_LUT_VERSION					-1038		/*!< correction lut version is not as expected */
#define	ERR_STEP_LESS_OR_EQUAL_ZERO			-1039		/*!< step is less or equal zero */
#define	ERR_STEP_TOO_BIG					-1040		/*!< widht *height / step is greater than 64*256 */
#define ERR_MAX_MODULE_REACHED_MODE0		-1041		/*!< maximum number of module reached in mode 0 */
#define ERR_MAX_MODULE_REACHED_MODE1		-1042		/*!< maximum number of module reached in mode 1 */
#define	ERR_GAIN_RANGE						-1043		/*!< gain is not between 0 and 15 */
#define	ERR_VAL_LESS_THAN_ZERO				-1044		/*!< value is less than zero */
#define	ERR_VAL_GREATER_THAN_MAX			-1044		/*!< value is less than expected */
#define	ERR_CONNECTION_NOT_ESTABLISHED		-1045		/*!< no connection to the camera established */
#define	ERR_PARAMETERFILE_NOT_FOUND			-1046		/*!< Parameterfile not found */
#define	ERR_NULL_ARG						-1047		/*!< NULL pointer for argument */
#define	ERR_PARSET_INVALID					-1048		/*!< Invalid Parameterset */
#define ERR_ROW_RANGE						-1049		/*!< Row number out of range */				
#define ERR_STRLEN_RANGE					-1050		/*!< String Length out of range */	


/**
 *	Special defines for Errors for the Get
 */
#define ERR_MODULE_ORDER					-10000		/*!< Module Order: Name not found! */
#define	ERR_MODULE_GET_NEXT_LINE			-10001		/*!< Module: Get Next Line, Reference Name Not Found */
#define	ERR_MODULE_TRACKING					-10002		/*!< Profile Tracking, Reference Name Not Found */
#define	ERR_MODULE_CORRELATOR				-10003		/*!< Correlator, Reference Profiles File not found */
#define	ERR_MODULE_PROFILE					-10004		/*!< Profile Trigger, Reference Name Not Found */
#define	ERR_MODULE_CALCULATOR				-10005		/*!< Module: Calculator, Reference Name Not Found */
#define	ERR_MODULE_COMPARE					-10006		/*!< Module: Compare, Reference Name Not Found */
#define	ERR_MODULE_GET_VAL					-10007		/*!< Module: Get Val, Reference Name Not Found */
#define	ERR_MODULE_LOGIC					-10008		/*!< Module: Logic, Reference Name Not Found */
#define	ERR_MODULE_READ_INPUT				-10009		/*!< Read Input, Reference Name Not Found */
#define	ERR_MODULE_TCP						-10010		/*!< Module: Send TCP, Reference Name Not Found */
#define	ERR_MODULE_CMOS_ROI					-10011		/*!< Module: CMOS Setup, ROI Value < 0 */
#define	ERR_MODULE_CMOS_STARTX				-10012		/*!< Module: CMOS Setup, StartX and Width must be a multiple of 8(32) */
#define	ERR_MODULE_CMOS_SIZE				-10013		/*!< Module: CMOS Setup, ROI  Exceeds Sensor Size */
#define	ERR_MODULE_CMOS_GAIN				-10014		/*!< Module: CMOS Setup, Gain Exceeds Limits 0..15 */


#endif
