/*============================================================================*/
/*                        SmartRay Camera API                                 */
/*----------------------------------------------------------------------------*/
/*    Copyright (c) SmartRay GmbH 2013.  All Rights Reserved.		          */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/* Title:       SR_API_public                                                 */
/*                                                                            */
/* Version:	SR_API_VERSION                                                    */
/* Date:	SR_API_VERSION                                                    */
/*                                                                            */
/*============================================================================*/
#ifndef SR_API_PUBLICh
#define SR_API_PUBLICh

#define APIVERSION "v3.6.21.0 b13091109\0"

#ifdef BUILD_DLL
// the dll exports
#define EXPORTDLL __declspec(dllexport)
//#define EXPORTDLL
#else
//#define EXPORTDLL __stdcall
#define EXPORTDLL __declspec(dllimport)
#endif


// Maximum Number of 3D Cameras
#define INSTANCES 6

// Size of TCP Receiver Buffers
#define TCPRECVSIZE 10000000

// States of Camera-Connection-Management
#define STATE_CAMERA_CLOSED 	0
#define STATE_CONNECT_TO_CAMERA 10  
#define STATE_CONNECT_TO_CAMERA_DELAYED 20  
#define STATE_CAMERA_CONNECTED	30  

// Default IP Setup of Camera
#define DEFAULT_IP_ADR "192.168.178.200"
#define DEFAULT_PORT_NUM 40

// List of Arguments for the Message Function 
#define STATUS_MSG_ARGLST CAMDESC* cd, int msgType, int msgdata1, int msgdata2, char *msg

// Status Meldungen
// Typ
#define MSGTYPE_CONNECTION		1
#define MSGTYPE_INFO			2
#define MSGTYPE_ERROR			3
#define MSGTYPE_DATA			4
// Gruppe
#define MSGGROUP_STATUS			1
#define MSGGROUP_SW_APPLICERROR 2
#define MSGGROUP_HW_APPLICERROR 3
#define MSGGROUP_API			4

// Datentyp
#define DATATYPE_IO				1
#define DATATYPE_CAMTEMP  		3
#define DATATYPE_SYSPAR	 		10
#define DATATYPE_MPAR    		11

// ERROR Meldungen
#define MSGGROUP_API_CORRUPTNUMOFCAMS			1
#define MSGGROUP_API_INVALIDFILENAME			2
#define MSGGROUP_API_CORRUPTPARAMETER			3
#define MSGGROUP_API_NOREFLEXPARAMETERFILE	4

#define MSGGROUP_STATUS_TCPTIMEOUT				1   /*from API*/
#define MSGGROUP_STATUS_TCPSENDERROR			2   /*from API*/ 
#define MSGGROUP_STATUS_TCPRECVERROR			4   /*from Camera*/
#define MSGGROUP_STATUS_FLASHWRITEERROR			5
#define MSGGROUP_STATUS_FLASHERASEERROR			6
#define MSGGROUP_STATUS_NOPARAMSLOADED			7

#define MSGGROUP_SW_APPLICERROR_BUFFER_OVERFLOW		1

#define MSGGROUP_HW_APPLICERROR_FIFO_OVERFLOW		2
#define MSGGROUP_HW_APPLICERROR_CORRELATOR_OVERFLOW 3
#define MSGGROUP_HW_APPLICERROR_CORRELATOR_SPEED	4
#define MSGGROUP_HW_APPLICERROR_UNKNOWN				1000

// INFO Messages
#define CAMERA_DISCONNECTED					0
#define CAMERA_CONNECTED						1

#define MSGGROUP_API_INITIALIZED				1

#define MSGGROUP_STATUS_REPLY					1
#define MSGGROUP_STATUS_PARAMS_RECEIVED			2
#define MSGGROUP_STATUS_PARAMS_FLASHWRITTEN		3

// 3D float (32bit) (real world point)
typedef struct
{
	float x;
	float y;
	float z;
} SR_3DPOINT;

// Camera Description
#define HEADERSIZE_USER	9
#define ALIVE_TIMEOUT	10.0 //[sec]
EXPORTDLL typedef struct 
{
	int cam_index;
	char name[260];
	char IPAdr[17];		// IP Adr String "192.168......
	unsigned short portnum;
	int command;
	unsigned char header[HEADERSIZE_USER];
	void* pcamdata;
	int camdatasize;
	int dataavailable;
	int connectionstate;
	int tcp_handle;
	int active;
	int alive_time;
	int archiv_active;
	int archiv_handle;
	int acknowledge;
	void *lut;
	void *usercbf;     //=cbf(CAMDESC* cd)
	int digio_out[4];
	int digio_in[4];
	int laser_status;
	int laserlight;
	int fps;          //Imager FramesPerSecond
	int tps;					//bit[15..0] Profil TriggerPerSecond  bit[31]=TriggerOverflow bit !!! 
	float Temp;
	int running;
} CAMDESC;

typedef struct
{
	int x;
	int y;
	int z;
} SRPIXEL;


//LUT World Values for not valid Profile Points. 
// all values <= NONWORLDMARK are not valid
#define NONWORLDMARK  -999990.0   
#define NONWORLDVALUE -999997.0
#define NONLUTVALUE   -999998.0
#define BADWORLDVALUE -999999.0


//---------------------------------------------------------------------------------------------------
#ifdef __cplusplus
    extern "C" {
#endif

// Funktionen

/** 
 *	@brief The return value is the version number of the API
 *  @param address of a character pointer
 *	@return SUCCESS, version string returned as function argument
 */
EXPORTDLL int SR_API_GetAPIVersion (char** version);

/** 
 *	@brief	Initialize the API 
 *  @param	cd - camera descriptor
 * 	@param	msgType - type of the message
 *  @param	msgdata1 - message data 1
 *  @param	msgdata2 - message data 2
 *	@param	msg - pointer of the message string
 * 	@return ERR_API_IS_INITIALIZED - API is successful not initialized
 *	@return	SUCCESS - successful
 */
EXPORTDLL int SR_API_Init (int (*userCB_StatusMessage) (STATUS_MSG_ARGLST));

/**
 *	@brief	function zu close the API
 *  @param	void
 *  @return SUCCESS - successful
 */
EXPORTDLL int SR_API_Exit (void);

/**
 *	@brief	start to initialized at every start an instance of a camera
 *	@param	cd - camera descriptor of the camera which should connected
 *  @return ERR_CD_NULLPOINTER - camera descriptor is a Nullpointer
 *  @return ERR_TOO_MANY_NR_OF_CAM_INSTANCES - maximum number of instances is reached
 *	@return ERR_ALLOCATION_FAILED - allocate receive buffer failed
 *	@return	ERR_CAM_STILL_ACTIVE - camera is still active
 *	@return SUCCESS - successful
 */
EXPORTDLL int SR_API_StartCameraConnectionManagement (CAMDESC* camdescription);

/**
 *	@brief 	Stop the camera
 *  @param 	cd - camera descriptor of the camera which should be stopped
 *  @return ERR_CD_NULLPOINTER - camera descriptor is a nullpointer 
 *  @return ERR_LESS_NR_OF_CAM_INSTANCES - number of instances of the cameras are lesser than zero
 * 	@return ERR_CAMERA_CONN_STILL_OPEN - the camera connection can't be closed
 *	@return	ERR_CAM_NOT_ACTIVE - the camera is not active
 *  @return SUCCESS - successful
 */
EXPORTDLL int SR_API_StopCameraConnectionManagement (CAMDESC* camdescription);

/**
 *	@brief	start the camera
 *	@param	cd - camera descriptor of the cam which should be started
 *	@return	ERR_CD_NULLPOINTER - camera descriptor is a nullpointer
 *	@return	@see SR_API_CommandSend
 */
EXPORTDLL int SR_API_StartCam (CAMDESC* cd);

/**
 *	@brief stops the camera
 * 	@param cd - camera descriptor which should be stopped
 * 	@return ERR_CD_NULLPOINTER - camera descriptor is a nullpointer
 * 	@return ERR_CAM_NOT_CONNECTED - camera is not connected 
 * 	@return ERR_CAM_NOT_ACTIVE - camera is not active
 * 	@return @see SR_API_CommandSend
 */
EXPORTDLL int SR_API_StopCam (CAMDESC* cd);

#define FILENAMESIZE	260   

/**
 * 	@brief	read parameterset from file
 *	@param	filename - path to the file which should be read
 * 	@return ERR_FILE_NOT_READ - file couldn't be read
 * 	@return ERR_SIZE_MISSMATCH - size of the read file doesn't match with the expected size
 * 	@return SUCCESS - successful
 */
EXPORTDLL int SR_API_ReadCamParsFromFile (char* filename);


/**
 *	@brief sending the parameterset to camera
 *	@param cd - camera descriptor of the camera which should receive the parameterset
 *	@return Errorcode
 *			cd is a nullpointer - "no camera object"
 *			@see SR_API_GetErrorString 
 */
EXPORTDLL int SR_API_SendParsToCam (CAMDESC* cd); 

/**
 *	@brief	checks the file extension and startet the appropriate function   
 *	@param 	cd - camera descriptor of the selected camera
 *	@param 	FileName - filename of the file which should update the camera
 *	@return	ERR_CD_NULLPOINTER - camera descriptor is a nullpointer
 *	@return	ERR_NULLPOINTER - FileName is a nullpointer
 *	@return	ERR_INVALID_FILENAME - invalid filename
 *	@return	ERR_INVALID_FILETYPE - camera doesn't match with the update file type
 *	@return	ERR_INVALID_UPDATEFILE - file is not a smartray update file
 *	@return	SUCCESS - successful
 */
EXPORTDLL int SR_API_Update_Cam (CAMDESC* cd, char* filename);

#define IPADRSIZE		4

/**
 *	@brief change IP number
 *	@param cd - camera descriptor for which the IP number should be change
 *	@param newip - pointer to the byte array which stores the new IP address
 *	@param newport - new port number on which camera should listen
 * 	@return ERR_CD_NULLPOINTER - camera descriptor is a nullpointer
 *	@return ERR_NO_SYSPAR_AVAILABLE - selected camera is not available
 *	@return ERR_IP_OR_PORT_IS_ZERO - newport or new IP are zero
 * 	@return ERR_IP_IS_BROADCAST - newip contain 4 times the value 255
 *	@return @see SR_API_CommandSend
 */
EXPORTDLL int SR_API_ChangeIp (CAMDESC* cd, unsigned char* newipadr, unsigned short newport);

#define MACSIZE			6

/** 
 *	@brief read the MAC address from the MPAR
 * 	@param cd - camera descriptor for which we want to get the MAC adress
 *	@param mac - pointer of a byte array to store the MAC address
 *	@return	ERR_CD_NULLPOINTER - camera descriptor is a nullpointer
 *	@return	ERR_NO_MPAR_AVAILABLE - no MPAR for the selected camera available
 *	@return	ERR_NULLPOINTER - mac is a nullpointer
 *	@return SUCCESS - successful
 */
EXPORTDLL int SR_API_GetMACAdr (CAMDESC* cd, unsigned char* mac);

#define SERIALSIZE		5

/**
 * 	@brief read the serialnumber from the MPAR
 * 	@param cd - camera descriptor for which we want to get the serial number
 * 	@param serial - pointer of an byte array to store the serial number
 *	@return	ERR_CD_NULLPOINTER - camera descriptor is a nullpointer
 *	@return	ERR_NO_MPAR_AVAILABLE - no MPAR for the selected camera available
 *	@return	ERR_NULLPOINTER - serial is a nullpointer
 *	@return SUCCESS - successful
 */
EXPORTDLL int SR_API_GetSerial (CAMDESC* cd, unsigned char* serial);

#define CAMVERSIONSIZE	64


/**
 * 	@brief	read the version information of the camera
 *		
 *			read the hardware and software version of the selected camera
 *	@param	cd - camera descriptor of the selected camera
 * 	@param	version_sw - pointer to the byte array in which the software version should be store
 *	@param	version_hw - pointer to the byte array in which the hardware version should be store
 * 	@return	ERR_CD_NULLPOINTER - camera descriptor is a nullpointer
 *	@return	ERR_NO_MPAR_AVAILABLE - no MPAR for the selected camera available
 *	@return	ERR_NULLPOINTER - version_sw is a nullpointer
 *	@return	ERR_NULLPOINTER - version_hw is a nullpointer
 *	@return	SUCCESS - successful
 */
EXPORTDLL int SR_API_GetCamVersions (CAMDESC* cd, char* version_sw, char* version_hw);


#define PARTNUMBERSIZE 16


/**
 * 	@brief	read partnumber to identify the sensor type
 *	@param	cd - camera descriptor of the selected camera
 *	@param	partNumber - stores the partnumber of the sensor
 *	@return ERR_CD_NULLPOINTER - camera descriptor is a nullpointer
 *	@return ERR_NULLPOINTER - partNumber is a nullpointer
 *	@return ERR_NO_MPAR_AVAILABLE - selected camera is not avaialbe
 *	@return SUCCESS - successful
 */
EXPORTDLL int SR_API_GetSensorType (CAMDESC* cd, char* partNumber);
// only use if working without sensor!

/**
 *	@brief  (Obsolete!!! See SR_API_GetImagerSize function) - Read size of the imager - 
 *	@param	cd - camera descriptor for the selected camera 
 *	@param	width - pointer for the storage of the imager width
 *	@param	height - pointer for the storage of the imager height
 *	@return ERR_WIDTH_IS_ZERO - width is zero 
 *	@return ERR_HEIGHT_IS_ZERO - height is zero
 *	@return	ERR_CD_NULLPOINTER - camera descriptor is a nullpointer
 *	@return	SUCCESS - successful 
 */	
EXPORTDLL int SR_API_GetSensorSize (CAMDESC* cd, int* width, int* height);

/**
*	@brief Read size of the imager 
*	@param	cd - camera descriptor for the selected camera 
*	@param	width - pointer for the storage of the imager width
*	@param	height - pointer for the storage of the imager height
*	@return ERR_WIDTH_IS_ZERO - width is zero 
*	@return ERR_HEIGHT_IS_ZERO - height is zero
*	@return	ERR_CD_NULLPOINTER - camera descriptor is a nullpointer
*	@return	SUCCESS - successful 
 */

EXPORTDLL int SR_API_GetImagerSize (CAMDESC* cd, int* width, int* height);


/**
 *	@brief	read the center position of the sensor
 * 	@param	cd - camera descriptor of the selected camera
 *	@param	centerX - midview of the X-axis
 *	@param	centerY - midview of the Y-axis
 *	@return ERR_CENTERX_RANGE_ERROR - centerX is not between 0 and 1280
 *	@return ERR_CENTERY_RANGE_ERROR - centerY is not between 0 and 1280
 *	@return ERR_CD_NULLPOINTER - camera descriptor is a nullpointer
 *	@return ERR_NULLPOINTER - centerX is a nullpointer
 *	@return ERR_NULLPOINTER - centerY is a nullpointer
 *	@return SUCCESS - successful
 */
EXPORTDLL int SR_API_GetSensorCenterPosition (CAMDESC* cd, int* centerX, int* centerY);

// Laser settings
// pulse_modes
#define LASER_PULSED      0
#define LASER_CONTINOUS   1
// external_mode
#define LASER_SW_ONLY     0
#define LASER_EXT_ON      1
// laser on/off
#define LASER_ENABLE      1
#define LASER_DISABLE     0
// use current setting
#define LASER_DONOTCHANGE 99


/**
 *	@brief configure the laser
 * 		2010-11-08: int power (0..100[%]) hinzu
 *	@param cd - camera descriptor of the selected camera
 *	@param laser_mode - select the mode of the laser
 *		0 - pulse
 *		1 - continous
 *	@param laser_on_select - control with software or hardware
 *		0 - software
 *		1 - soft+extern
 *	@param enable - enable oder disable the laser
 * 	@param power - power of the laser, values between 0 and 100%
 * 	@return ERR_CD_NULLPOINTER - camera descriptor is a nullpointer
 *	@return @see SR_API_CommandSend
 */
EXPORTDLL int SR_API_SetLaser (CAMDESC* cd, int pulse_mode, int external_mode, int enable, int power); //power: 0..100 [%]

#define IO_HIGH		1
#define IO_LOW		0


/**
 * 	@brief	set the digital output of the camera
 *	@param	cd - camera descriptor of the selected camera
 *	@param	channel - select the channel which should be configure
 *	@param	val - value to which the selected channel should be set
 * 	@return ERR_CD_NULLPOINTER - camera descriptor is a nullpointer
 *	@return @see SR_API_CommandSend 
 */
EXPORTDLL int SR_API_SetOutput (CAMDESC* cd, int channel, int val);



/**
 *	@brief get input status from the camera
 *		
 *		this will be done only local from API because the information will be update
 *		every 200ms via the alive signal
 * 	@param cd - camera descriptor of the selected camera
 *	@param channel - selected channel 
 *	@param val - pointer for the storage of the selected channel status
 * 	@return ERR_CD_NULLPOINTER - camera descriptor is a nullpointer
 * 	@return ERR_NULLPOINTER - val is a nullpointer
 *	@return SUCCESS - successful
 */
EXPORTDLL int SR_API_GetInput (CAMDESC* cd, int channel, int* val);


// dattyp defines
#define DATATYPE_RAWIMAGE				    	0
#define DATATYPE_PROFILE				    	1
#define DATATYPE_INTENSITY				    	2
#define DATATYPE_PROFILE_PLUS_INTENSITY	    	3
#define DATATYPE_ADVANCED				    	4
#define DATATYPE_ADVZC_PROFILE   				5
#define DATATYPE_ADVANCED_HDR					6
#define DATATYPE_ADVZC_PROFILE_PLUS_INTENSITY  	7
#define DATATYPE_ADVZC_INTENSITY 				8
#define DATATYPE_IOSTATUS				    	255



// UserCB Types	
#define SRCB_RAWDATAMODE		0 
typedef int (*ucbf_rawdatamode) (CAMDESC* cd);
#define SRCB_CAMDATAMODE		1
typedef int (*ucbf_camdatamode) (CAMDESC* cd, int dattyp, int sx, int height, int width, void* pdata);
#define SRCB_CAMDATAEXTMODE	2
typedef int (*ucbf_camdataextmode) (CAMDESC* cd, int dattyp, int sx, int numlines, int len_data, void* data, int len_extdata, void* extdata);
#define SRCB_SENSORDEFAULTMODE	10
typedef int (*ucbf_sensordefaultmode) (CAMDESC* cd, int dattyp, int sx, int numlines, int len_data, void* profile,void* intensity,void* llt, int numExtData, void* extdata);



// Maximale Anzahl an CBs
#define MAXNUM_SRCB_RAWDATA 	32
#define MAXNUM_SRCB_CAMDATA 	8
#define MAXNUM_SRCB_CAMDATAEXT  8
#define MAXNUM_SRCB_PIXELMODE 	2
#define MAXNUM_SRCB_SENSORDEFAULTMODE 8

// nicht verfgbare Kamerakanle
#define CAMCB_RESERVED 0x2000 - 0x2fff

// Standard KameraDatenkanle (zu verwenden in SR_API_RegisterUserCB)
#define DEFAULT_LIVEIMAGE_CHANNEL		0x8010
#define DEFAULT_PROFILEIMAGE_CHANNEL	0x8020



/**
 *	@brief	define user callbacks for special functionality
 *	@param	CBtype - type of the callback function
 *	@param	command - command which should be send from the callback
 * 	@param	userCB - pointer to the callback function which should called
 *	@return ERR_NR_USERCB_RAWDATA_MAX - numUserCB_RawData is equal to MAXNUM_SRCB_RAWDATA
 *	@return ERR_NR_USERCB_CAMDATA_MAX - numUserCB_CamData is equal to MAXNUM_SRCB_CAMDATA
 *	@return ERR_NR_USERCB_CAMDATAEXT_MAX - numUserCB_DataExt is equal to MAXNUM_SRCB_CAMDATAEXT
 * 	@return ERR_NR_USERCB_PIXELMODE_MAX - numUserCB_PixelMode is equal to MAXNUM_SRCB_PIXELMODE
 *	@return ERR_NO_VALID_CB_TYPE - no valid CBtype
 *	@return	SUCCESS - successful
 */
EXPORTDLL int SR_API_RegisterUserCB (int CBtype, int command, void *userCB);    


// Change Camera Parameter Variables with a Function Call -----------------------------------------
// 1st read parameters from file use:		SR_API_ReadCamParsFromFile
// 2nd send parameters to cam    use:   SR_API_SendParsToCam
// 3rd use this functions to change values in Camera


/**
 *	@brief	configure the exposure module and send it to cam
 *	@param 	cd - camera descriptor of the selected camera
 *	@param 	inst - instance of module
 *	@param 	enableDoubleExpo - enable double exposure
 *	@param 	enableCtrlSmall - enable the short exposure time
 *	@param 	enableCtrlLarge - enable the large exposure time
 *	@param 	expo_small - time of the short exposure time in microseconds
 *	@param 	expo_large - time of the large exposure time in microseconds
 *	@return	ERR_CD_NULLPOINTER - camera descriptor is a nullpointer
 *	@return @see SR_API_CommandSend
 */
EXPORTDLL int SR_API_SetExposure (CAMDESC* cd, int inst, 
												int enableDoubleExpo,
												int expo_small,
												int expo_large);


/**
 *	@brief	configure the trigger module and send it to camera
 *	@param 	cd - camera descriptor of the selected camera
 *	@param 	inst - instance of module
 *	@param 	mode - set the trigger mode
 *			0 - free running
 *			1 - intern
 *			2 - extern
 *			3 - extern In 0 enabled
 *			4 - In 0 enabled
 *	@param 	source - set the trigger source
 *			0 - In 1
 *			1 - In 2
 *			2 - both
 *			3 - intern (TTL)
 *			4 - quad
 *	@param 	edge - on which edge should be trigger
 *			0 - rising
 *			1 - falling
 *			2 - both
 *	@param 	outputselect - 
 *				- Out0 = bit[0]
 *				- Out1 = bit[1]
 *	@param 	trigfrq - frequency of the trigger
 *	@param 	trigcnt - count of suppressed trigger (trigger divider)
 *	@param 	trigoffset - number of suppressed trigger after start camera
 *	@return	ERR_CD_NULLPOINTER - camera descriptor is a nullpointer
 *	@return @see SR_API_CommandSend
 */

EXPORTDLL int SR_API_SetTrigger(CAMDESC* cd, int inst, 
											 int mode,
											 int source,
											 int edge,
											 int outputselect,		// Trigger Out0=bit[0] Out1=bit[1]
											 int trigfrq,
											 int trigcnt,
											 int trigoffset);


/**
 *	@brief	configure the CMOS module and send it to camera
 *	@param 	cd - camera descriptor of the selected camera
 *	@param 	inst - instance of module
 *	@param 	startx - start position of the ROI in x direction
 *	@param 	starty - start position of the ROI in y direction
 *	@param 	width - width of the ROI
 *	@param 	height - height of the ROI
 *	@param 	gainEnable - enable the gain of the CMOS imager 
 *	@param 	gain - configure the gain of the imager
 *	@return	ERR_CD_NULLPOINTER - camera descriptor is a nullpointer
 *	@return	ERR_GAIN_RANGE - gain is not between 0 and 15
 *	@return	ERR_VAL_LESS_THAN_ZERO - starty, startx, width or height is less than zero
 *	@return	ERR_RANGE_CHECK - step size in x direction of the imager and the startx position or the width  
 *					of the ROI have a mismatch 
 *	@return	ERR_RANGE_CHECK - imager width and the sum of start position and width of the ROI have a mismatch
 *	@return	@see SR_API_CommandSend
 */
EXPORTDLL int SR_API_SetImager(CAMDESC* cd, int inst, int startx, int starty, int width, int height, int gainEnable, int gain);


/**
 * @brief	configure the READ_INPUT module and send it to camera
 * @param 	cd - camera descriptor of the selected camera
 * @param 	inst - instance of module
 * @param 	activ[0..3] - 1 - if IN_[0..3] is activ  
 *                        0 - if IN_[0..3  is not used
 * @param 	condition[0..3] - 1 - if IN_[0..3] triggers with POSITIV edge
 *                            0 - if IN_[0..3] triggers with NEGATIV edge 
 * @return	0	- successful
 *          -1 - Parameters not loaded from file
 *          -2 - inst out of range 0..3
 *          -3 - Analyse Parameter Error
 */

EXPORTDLL int SR_API_SetReadInput (CAMDESC* cd, int inst, int *active, int *condition);


/**
 * @brief	read the CMOS module from the loaded parameterset
 * @param 	cd - camera descriptor of the selected camera
 * @param 	inst - instance of module
 * @param 	*startx - start position of the ROI in x direction
 * @param 	*starty - start position of the ROI in y direction
 * @param 	*width - width of the ROI
 * @param 	*height - height of the ROI
 * @param 	*gainEnable - enable the gain of the CMOS imager 
 * @param 	*gain - configure the gain of the imager
 * @return	0	- successful
 *          -1 - no parameter file loaded
 *          -2 - inst out of range	0..1									
 */

EXPORTDLL int SR_API_GetImager(CAMDESC* cd, int inst, int *startx, int *starty, int *width, int *height, int *gainEnable, int *gain);

/**
 * @brief	read the Imager Granularity
 * @param 	cd - camera descriptor of the selected camera
 * @param 	*stepSizeX - pointer to stepSizeX of the imager
 * @param 	*stepSizeY - pointer to stepSizeY of the imager
 * @return	0	- successful
 *          -1 - ERR_CD_NULLPOINTER
 */

EXPORTDLL int SR_API_GetImagerGranularity (CAMDESC* cd, int* stepSizeX, int* stepSizeY);

/**
 * @brief   read the exposure module from the loaded parameterset
 * @param 	cd - camera descriptor of the selected camera
 * @param 	inst - instance of module
 * @param 	*enableDoubleExpo - enable double exposure
 * @param 	*enableCtrlSmall - enable the short exposure time
 * @param 	*enableCtrlLarge - enable the large exposure time
 * @param 	*expo_small - time of the short exposure time in microseconds
 * @param 	*expo_large - time of the large exposure time in microseconds
 * @return  0 - successful
 */

EXPORTDLL int SR_API_GetExposure (CAMDESC* cd, int inst, int *enableDoubleExpo,	int *expo_small, int *expo_large);



/**
 * @brief   read the trigger module from the loaded parameterset
 * @param 	cd - camera descriptor of the selected camera
 * @param 	inst - instance of module
 * @param 	*mode - set the trigger mode
 *			0 - free running
 *			1 - intern
 *			2 - extern
 *			3 - extern In 0 enabled
 *			4 - In 0 enabled
 * @param 	*source - set the trigger source
 *			0 - In 1
 *			1 - In 2
 *			2 - both
 *			3 - intern (TTL)
 *			4 - quad
 * @param 	*edge - on which edge should be trigger
 *			0 - rising
 *			1 - falling
 *			2 - both
 * @param 	*outputselect - 
 *				- Out0 = bit[0]
 *				- Out1 = bit[1]
 * @param 	*trigfrq - frequency of the trigger
 * @param 	*trigcnt - count of suppressed trigger (trigger divider)
 * @param 	*trigoffset - number of suppressed trigger after start camera
 * @return  0 - successful
 *          -1 - no parameter file loaded
 *          -2 - inst out of range	0
 */

EXPORTDLL int SR_API_GetTrigger(CAMDESC* cd, int inst, int *mode,
													   int *source,
													   int *edge,
													   int *outputselect,		// Trigger Out0=bit[0] Out1=bit[1]
													   int *trigfrq,
													   int *trigcnt,
													   int *trigoffset);

/**
 * @brief	read the READ_INPUT module from the loaded parameterset 
 * @param 	cd - camera descriptor of the selected camera
 * @param 	inst - instance of module
 * @param 	activ[4]     - 1 - if IN_[0..3] is activ  
 *                         0 - if IN_[0..3  is not used
 * @param 	condition[4] - 1 - if IN_[0..3] triggers with POSITIV edge
 *                         0 - if IN_[0..3] triggers with NEGATIV edge 
 * @return	0	- successful
 *          -1 - Parameters not loaded from file
 *          -2 - inst out of range 0..3
 *          -3 - Analyse Parameter Error
 */

EXPORTDLL int SR_API_GetReadInput (CAMDESC* cd, int inst, int *active,
														  int *condition);


//Create World with float X,Y,Z World Data  | dimension [mm] 
//returns> 0:success  -1:No LUT loaded 


/**
 * @brief   create WorldCoordinates with LUT in SR_3DPOINT.float
 * @param 	cd - camera descriptor of the selected camera
 * @param 	startX - start position in x direction
 * @param 	width - width of the profile
 * @param 	profil - pointer for the storage of the profiles, profiles has a 14 bit width
 * @param 	world - pointer for the storage of the world coordinates as X,Y,Z Tupel
 * @return  ERR_CD_NULLPOINTER - camera descriptor is a nullpointer
 * @return	ERR_NULLPOINTER - pointer of the storage of the profiles is a nullpointer
 * @return	ERR_NULLPOINTER - pointer of the storage of the world coordinates is a nullpointer
 * @return	ERR_NULLPOINTER - LUT of the camera is a nullpointer
 * @return	ERR_OUT_OF_RANGE - startX and the width is greater than the LUT width
 * @return	-6 - index of the LUT is not valid
 * @return	SUCCESS - successful
 */
EXPORTDLL int SR_API_CreateWorldData (CAMDESC *cd, int startX, int width,	//-Pixel X Start, ProfilWidth  
										unsigned short *profil, //-Profil as it sent from the Cam
										SR_3DPOINT *world);		//- Profil transformed to Worldkoord. as X,Y,Z  triplets

																	
// lut info header string read to get NAME, DATE ... of the lut stored in the camera
// You must provide an info-Arrays of at least "LUTINFOSIZE" length
#define LUTINFOSIZE 	1024 

/**
 *	@brief	read the LUT information from file
 *	@param	cd - camera descriptor of the selected camera
 *	@param	DateiPfad - path of the LUT 
 *	@param	info - pointer of the storage for the read LUT information
 *	@return	ERR_CD_NULLPOINTER - camera descriptor is a nullpointer
 *	@return	ERR_NULLPOINTER - DateiPfad is a nullpointer
 *	@return	ERR_NULLPOINTER - info is a nullpointer
 *	@return	ERR_READ_FILE_FAILED - reading of the LUT information failed
 *	@return ERR_FILE_NEVER_FOUND - no file found
 *	@return	SUCCESS - successful
 */
EXPORTDLL int SR_API_ReadLutInfoFile (CAMDESC *cd, char *DateiPfad, char *infoLut);

/**
 *	@brief	load a LUT form file
 *	@param	cd - camera descriptor of the selected camera
 *	@param	DateiPfad - path of the LUT 
 *	@param	info - store LUT information
 *	@return	ERR_CD_NULLPOINTER - camera descriptor is a nullpointer
 *	@return	ERR_NULLPOINTER - DateiPfad is a nullpointer
 *	@return	ERR_NULLPOINTER - info is a nullpointer
 *	@return	ERR_SIZE_MISSMATCH - header size is to small
 *	@return	ERR_CAM_NOT_CONNECTED - no information of the imager size
 *	@return	ERR_CAM_NOT_CONNECTED - wrong imager width
 *	@return	ERR_CAM_NOT_CONNECTED - wrong imager height
 *	@return	ERR_ALLOCATION_FAILED - allocate memory for the LUT failed
 *	@return	ERR_SIZE_MISSMATCH - data size to small
 *	@return	SUCCESS - successful
 */
EXPORTDLL int SR_API_LoadLutFromFile (CAMDESC *cd, char *DateiPfad, char *info );  // LUT + 1024info - Daten aus einer Datei lesen und in LUT schreiben


/**
 *	@brief	save LUT as binary file
 *	@param	cd - camera descriptor of the selected camera
 *	@param	DateiPfad - path to save the LUT
 *	@param	info -
 *	@return	ERR_CD_NULLPOINTER - camera descriptor is a nullpointer
 *	@return	ERR_NULLPOINTER - LUT of the camera descriptor is a nullpointer
 *	@return	ERR_FILE_NEVER_FOUND - no file found
 *	@return	ERR_WRITE_FILE_FAILED - write information to file failed
 *	@return	ERR_WRITE_FILE - write LUT to file failed
 *	@return	SUCCESS - successful
 */
EXPORTDLL int SR_API_SaveLutToFile   (CAMDESC *cd, char *DateiPfad, char *info);  // LUT in eine Datei schreiben  incl 1024 byte INFO  

/**************************************************************************************************************************
	Function mapped world coordinates in a matrix, while maintaining the z-and y-world
	
	srW  			Input World data,  one profile
	intens			Input Intensity data
	len        		Length / width  of profile 
	range_start		Remaper Y world start coordinate
	range_end		Remaper Y world end coordinate
	iS				Result array for intensity profil (16bit)
	zS				Result array for World Z coordinate (16bit)
	offsetZ       	Value added to the world Z value before it is scaled 
	scalZ			Divider to convert the world z (float) value into a 16bit  =>(bei zW in mm -->1/100 == 10m Auflsung)
	
		
	returns> 0:success  -1:len>MAXREMAPDATALEN 	  -2:len<0
**************************************************************************************************************************/
#define MAXREMAPDATALEN 1296


/**
 *	@brief	remapping the world coordinates in to a matrix and keep z-world and y-world
 *			Erst wird zu WeltZ der offsetZ addiert, das Ergebnis wird dann mit scaleZ multipliziert
 *
 *		Version: 	1.1.0
 *		2011-01-20 	scaleZ ist nun 1/scaleZ.alt also 0.010 statt 100 eintragen
 *					index_last wird beim Lcke suchen auch inkremetiert !! 
 *		2011-01-20:	TA damit sind  Fehlfunktionen des Remappers (bis neue gefunden) eliminiert
 *	@param	srw - world coordinates
 *	@param	intens - intensity array after the separation in profils and intensity from the camera
 *	@param	len - length of the world coordinates which is equal to the number of intensity data
 *	@param	range_start - start position for the remapping of the world coordinates
 *	@param	range_end - end position for the remapping of the world coordinates
 *	@param	iS - pointer to store the intensity data after remapping 
 *	@param	zS - pointer to store the z coordinates after remapping
 *	@param	offsetZ - offset from z for the storage of the world into 16 bit
 *	@param	scaleZ - scale of the z direction for example 0.01 => zW in mm --> 1/100 == 10um resolution
 *	@return	ERR_VAL_LESS_THAN_ZERO - len is zero or less
 *	@return	ERR_VAL_GREATER_THAN_MAX - len is greater than MAXREMAPDATALEN
 *	@return	SUCCESS - successful
 */
EXPORTDLL int SR_API_RemapWorldToMatrix (SR_3DPOINT *srW, unsigned short *intens, int len, float range_start, float range_end, unsigned short *iS, unsigned short *zS, float offsetZ, float scaleZ);


/**
 * @brief	Create an error string out of a error code
 * @param 	error_code - error code for which we want to generate an error string
 * @param 	errstr - The error string pointer (pass a valid character pointer address)
 * @return 	errstr - Points the passed address to error string
 * @return	SUCCESS
 * @return	ERR_NULLPOINTER - Invalid character poiner address
 */
EXPORTDLL int SR_API_GetErrorString(int error_code, char** errstr);


/**
* @brief    read the module structure names from the loaded parameterset 
 * @param    cd - camera descriptor of the selected camera  
 * @param   name        - string with module name
* @param   methodeName - string with module methode name
* @param   methodeCode - methode Code number
+ @param   nextName    - string with name of next module
* @param    inst        - instance of module
* @param    numModule   - number of modules in parameter set 
*
* @return  SUCCESS - successful
*          ERR_CD_NULLPOINTER - camera descriptor not valid
*          ERR_PARSET_INVALID - parameter set not loaded (valid)
*          ERR_ROW_RANGE - rowNum out of range 1..max rowNum in parameter set
*          ERR_NULL_ARG - NULL pointer for argument
*/

EXPORTDLL int SR_API_GetModule (CAMDESC* cd,  int rowNum, char *name, char *methodName, int  *methodCode, char *nextName, int *inst, int *numModule);

/**
* @brief    read the module structure names from the loaded parameterset 
* @param    cd - camera descriptor of the selected camera  
* @param   rowNum        - Row number in the parameterset
* @param   nextName    - string with name of next module
* @return  SUCCESS - successful
*          ERR_CD_NULLPOINTER - camera descriptor not valid
*          ERR_PARSET_INVALID - parameter set not loaded (valid)
*          ERR_ROW_RANGE - rowNum out of range 1..max rowNum in parameter set
*          ERR_NULL_ARG - NULL pointer for argument
*          ERR_STRLEN_RANGE - strlen(nextName) out of range (>255 char)
*/

EXPORTDLL int SR_API_SetModule (CAMDESC* cd,  int rowNum, char *nextName);

/**
 *	@brief	Get the Sensor Name
 *	@param	cd - camera descriptor of the selected camera
 *	@param	productName - Product name of the selected camera
 * 	@return ERR_CD_NULLPOINTER - camera descriptor is a nullpointer
 * 	@return ERR_NULLPOINTER - Product name passed is a nullpointer
 *	@return SUCCESS
 */
EXPORTDLL int SR_API_GetSensorName (CAMDESC* cd, char* productName);

/**
* @brief    Set the GetNextLine & SendTCP module and send it to camera
* @param    cd - camera descriptor of the selected camera
* @param    inst - instance of module
* @param    packetSize - number of profile lines to transmit 
* @param   numProfiles  - stop sensor after this number of profile lines are generated     / if 0 (zero) we run continuously
*                             
* @return SUCCESS - successful
*          ERR_PARAMETERFILE_NOT_FOUND - Parameters not loaded from file
*          ERR_TOO_MANY_NR_OF_CAM_INSTANCES - inst out of range 0..3
*          
 */
EXPORTDLL int SR_API_SetParamProfile (CAMDESC* cd, int inst, int packetSize, int numProfiles);

/**
* @brief    Get values from the GetNextLine & SendTCP module and send it to camera
* @param    cd - camera descriptor of the selected camera
* @param    inst - instance of module
* @param    packetSize - number of profile lines to transmit 
* @param    numProfiles  - stop sensor after this number of profile lines are generated     / if 0 (zero) we run continuously
*                             
* @return SUCCESS - successful
*          ERR_PARAMETERFILE_NOT_FOUND - Parameters not loaded from file
*          ERR_TOO_MANY_NR_OF_CAM_INSTANCES - inst out of range 0..3
*              
 */

EXPORTDLL int SR_API_GetParamProfile (CAMDESC* cd, int inst, int *packetSize, int *numProfiles);


/**
 *	@brief   convert world points in to image points
 *
 *			Version      : 1.01
 *			Convert a WorldPoint (float) into a nearest Imager Position 
 *          with use of the LUT (no Subpixeling)
 *          We use a iteration to find the img. Point !!	 
 *          We limit the databuswidth to 10bit !! because we do not use subpix !!
 *          so the max ImagerHeight is 1024 pix !!*
 *			Input:  Wordcoordinate as X, Y,Z Tupel (X is not used)  just ONE point!
 *			Output: Corresponding Imager Coordinate to the given World Point
 *	@param 	cd - camera descriptor of the selected camera
 *	@param 	world - world coordinates we want to convert
 *	@param 	imgX - pointer for the storage of the image x coordinats
 *	@param 	imgY - pointer for the storage of the image y coordinats
 *	@return	ERR_CD_NULLPOINTER - camera descriptor is a nullpointer
 *	@return	ERR_NULLPOINTER - world is a nullpointer
 *	@return	ERR_NULLPOINTER - imgX is a nullpointer
 *	@return	ERR_NULLPOINTER - imgY is a nullpointer
 *	@return	ERR_FUNCTION_NOT_SUCCESSFUL - no success
 *	@return	ERR_NO_IMAGE_POINT_FOUND - image point found
 *	@return	ERR_NULLPOINTER - LUT of the camera is a nullpointer
 *	@return ERR_LUTWIDTH_IS_ZERO - LUTWidth is zero
 *	@return ERR_LUTHEIGHT_IS_ZERO - LUTHeight is zero
 *	@return	SUCCESS - successful
 */
EXPORTDLL int SR_API_WorldToImage	( CAMDESC *cd, SR_3DPOINT *world, int *imgX, int *imgY);

// Data Arrays
#define EXTMODE_BUFSIZE 10000000
#define EXTMODE_EXTDATASIZE 10000



#ifdef __cplusplus
    }
#endif


#endif
	
